@@ -101,6 +101,11 @@ class SGD
       const bool resetPolicy = true,
       const bool exactObjective = false);
 
+  /**
+   * Clean any memory associated with the SGD object.
+   */
+  ~SGD();
+
   /**
    * Optimize the given function using stochastic gradient descent.  The given
    * starting point will be modified to store the finishing point of the
